@@ -1,21 +1,14 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from trytond.model import ModelStorage, ModelView, fields
-from trytond.pool import PoolMeta, Pool
-from trytond.pyson import Eval
-
-__all__ = ['ClassificationMixin', 'classification_tree',
-    'Template', 'Product', 'ClassificationDummy', 'ClassificationTreeDummy']
+from trytond.model import (
+    DeactivableMixin, ModelStorage, ModelView, fields, tree)
+from trytond.pool import Pool, PoolMeta
 
 
-class ClassificationMixin(object):
+class ClassificationMixin(DeactivableMixin):
+    __slots__ = ()
     name = fields.Char('Name', translate=True, required=True)
-    active = fields.Boolean('Active', select=True)
-    selectable = fields.Boolean('Selectable', select=True)
-
-    @classmethod
-    def default_active(cls):
-        return True
+    selectable = fields.Boolean("Selectable")
 
     @classmethod
     def default_selectable(cls):
@@ -25,37 +18,33 @@ class ClassificationMixin(object):
 def classification_tree(name):
     'Return a ClassificationMixin with tree structure'
 
-    class ClassificationTreeMixin(ClassificationMixin):
-        parent = fields.Many2One(name, 'Parent', select=True)
+    class ClassificationTreeMixin(tree(separator=' / '), ClassificationMixin):
+        __slots__ = ()
+        parent = fields.Many2One(name, "Parent")
         childs = fields.One2Many(name, 'parent', 'Children')
 
-        @classmethod
-        def validate(cls, records):
-            super(ClassificationMixin, cls).validate(records)
-            cls.check_recursion(records, rec_name='name')
-
-        def get_rec_name(self, name):
-            if self.parent:
-                return self.parent.get_rec_name(name) + ' / ' + self.name
-            else:
-                return self.name
-
     return ClassificationTreeMixin
 
 
-class Template:
-    __metaclass__ = PoolMeta
+class Template(metaclass=PoolMeta):
     __name__ = 'product.template'
 
-    classification = fields.Reference('Classification',
-        selection='get_classification',
-        domain=[
-            ('selectable', '=', True),
-            ],
-        states={
-            'readonly': ~Eval('active', True),
-            },
-        depends=['active'])
+    classification = fields.Reference(
+        "Classification", selection='get_classification')
+
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        for model in cls._get_classification():
+            if model in cls.classification.domain:
+                cls.classification.domain[model] = [
+                    cls.classification.domain[model],
+                    ('selectable', '=', True),
+                    ]
+            else:
+                cls.classification.domain[model] = [
+                    ('selectable', '=', True),
+                    ]
 
     @classmethod
     def _get_classification(cls):
@@ -66,15 +55,12 @@ class Template:
     def get_classification(cls):
         pool = Pool()
         Model = pool.get('ir.model')
+        get_name = Model.get_name
         models = cls._get_classification()
-        models = Model.search([
-                ('model', 'in', models),
-                ])
-        return [(None, '')] + [(m.model, m.name) for m in models]
+        return [(None, '')] + [(m, get_name(m)) for m in models]
 
 
-class Product:
-    __metaclass__ = PoolMeta
+class Product(metaclass=PoolMeta):
     __name__ = 'product.product'
 
     @classmethod
